@@ -80,7 +80,6 @@ bool CServerPath::SetPath(std::wstring newPath)
 bool CServerPath::SetPath(std::wstring &newPath, bool isFile)
 {
 	std::wstring path = newPath;
-	std::wstring file;
 
 	if (path.empty()) {
 		return false;
@@ -205,20 +204,40 @@ bool CServerPath::HasParent() const
 
 CServerPath CServerPath::GetParent() const
 {
-	if (empty() || !HasParent()) {
-		return CServerPath();
-	}
-
 	CServerPath parent(*this);
-	CServerPathData& parent_data = parent.m_data.get();
+	parent.MakeParent();
+	return parent;
+}
 
-	parent_data.m_segments.pop_back();
+CServerPath& CServerPath::MakeParent()
+{
+	if (empty() || !HasParent()) {
+		clear();
+	}
+	else {
+		CServerPathData& data = m_data.get();
+		data.m_segments.pop_back();
 
-	if (m_type == MVS) {
-		parent_data.m_prefix = fz::sparse_optional<std::wstring>(L".");
+		if (m_type == MVS) {
+			data.m_prefix = fz::sparse_optional<std::wstring>(L".");
+		}
 	}
 
-	return parent;
+	return *this;
+}
+
+std::wstring CServerPath::GetFirstSegment() const
+{
+	if (empty() || !HasParent()) {
+		return std::wstring();
+	}
+
+	if (!m_data->m_segments.empty()) {
+		return m_data->m_segments.front();
+	}
+	else {
+		return std::wstring();
+	}
 }
 
 std::wstring CServerPath::GetLastSegment() const
@@ -321,7 +340,7 @@ bool CServerPath::DoSetSafePath(std::wstring const& path)
 	// from file
 	wchar_t const* const begin = path.c_str();
 	wchar_t const* const end = begin + path.size();
-	
+
 	wchar_t const* p = begin;
 
 	int type = 0;
@@ -449,14 +468,16 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 				return false;
 			}
 		}
-		if (!cmpNoCase && m_data->m_prefix != path.m_data->m_prefix)
+		if (!cmpNoCase && m_data->m_prefix != path.m_data->m_prefix) {
 			return false;
+		}
 	}
 
 	// On MVS, dirs like 'FOO.BAR' without trailing dot cannot have
 	// subdirectories
-	if (traits[m_type].prefixmode == 1 && !path.m_data->m_prefix)
+	if (traits[m_type].prefixmode == 1 && !path.m_data->m_prefix) {
 		return false;
+	}
 
 	tConstSegmentIter iter1 = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
@@ -821,12 +842,10 @@ bool CServerPath::operator!=(const CServerPath &op) const
 bool CServerPath::operator<(const CServerPath &op) const
 {
 	if (empty()) {
-		if (!op.empty()) {
-			return false;
-		}
+		return !op.empty();
 	}
 	else if (op.empty()) {
-		return true;
+		return false;
 	}
 
 	if (m_data->m_prefix || op.m_data->m_prefix) {
@@ -910,17 +929,22 @@ std::wstring CServerPath::FormatFilename(std::wstring const& filename, bool omit
 
 int CServerPath::CmpNoCase(const CServerPath &op) const
 {
-	if (empty() != op.empty())
+	if (empty() != op.empty()) {
 		return 1;
-	else if (m_data->m_prefix != op.m_data->m_prefix)
+	}
+	else if (m_data->m_prefix != op.m_data->m_prefix) {
 		return 1;
-	else if (m_type != op.m_type)
+	}
+	else if (m_type != op.m_type) {
 		return 1;
+	}
 
-	if (m_data->m_segments.size() > op.m_data->m_segments.size())
+	if (m_data->m_segments.size() > op.m_data->m_segments.size()) {
 		return 1;
-	else if (m_data->m_segments.size() < op.m_data->m_segments.size())
+	}
+	else if (m_data->m_segments.size() < op.m_data->m_segments.size()) {
 		return -1;
+	}
 
 	tConstSegmentIter iter = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = op.m_data->m_segments.begin();
@@ -995,8 +1019,9 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 		}
 		parentData.m_prefix = GetParent().m_data->m_prefix;
 	}
-	else
+	else {
 		parentData.m_prefix = m_data->m_prefix;
+	}
 
 	tConstSegmentIter iter = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
@@ -1038,13 +1063,10 @@ bool CServerPath::SegmentizeAddSegment(std::wstring & segment, tSegmentList& seg
 			return true;
 		}
 		else if (segment == L"..") {
-			if (segments.empty()) {
-				return false;
-			}
-			else {
+			if (!segments.empty()) {
 				segments.pop_back();
-				return true;
 			}
+			return true;
 		}
 	}
 
