@@ -22,6 +22,10 @@
 #include "volume_enumerator.h"
 #endif
 
+#ifdef USE_MAC_SANDBOX
+#include "osx_sandbox_userdirs.h"
+#endif
+
 #include <algorithm>
 
 class CTreeItemData : public wxTreeItemData
@@ -31,7 +35,7 @@ public:
 	std::wstring m_known_subdir;
 };
 
-class CLocalTreeViewDropTarget : public CScrollableDropTarget<wxTreeCtrlEx>
+class CLocalTreeViewDropTarget final : public CScrollableDropTarget<wxTreeCtrlEx>
 {
 public:
 	CLocalTreeViewDropTarget(CLocalTreeView* pLocalTreeView)
@@ -91,38 +95,41 @@ public:
 		}
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
-		if (!hit)
+		if (!hit) {
 			return wxDragNone;
+		}
 
 		const CLocalPath path(GetDirFromItem(hit).ToStdWstring());
-		if (path.empty() || !path.IsWriteable())
+		if (path.empty() || !path.IsWriteable()) {
 			return wxDragNone;
+		}
 
-		if (!GetData())
+		if (!GetData()) {
 			return wxDragError;
+		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager)
+		if (pDragDropManager) {
 			pDragDropManager->pDropTarget = m_pLocalTreeView;
+		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
+		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
 			m_pLocalTreeView->m_state.HandleDroppedFiles(m_pFileDataObject, path, def == wxDragCopy);
-		else
-		{
-			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
-			{
+		}
+		else {
+			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalTreeView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalTreeView->m_state.GetServer()))
-			{
+			if (!m_pLocalTreeView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pLocalTreeView->m_state.GetServer().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalTreeView->m_state.DownloadDroppedFiles(m_pRemoteDataObject, path))
+			if (!m_pLocalTreeView->m_state.DownloadDroppedFiles(m_pRemoteDataObject, path)) {
 				return wxDragNone;
+			}
 		}
 
 		return def;
@@ -216,8 +223,6 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CLocalTreeView, wxTreeCtrlEx)
-
 BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CLocalTreeView::OnItemExpanding)
 #ifdef __WXMSW__
@@ -273,6 +278,13 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, C
 	SetItemImage(root, GetIconIndex(iconType::dir), wxTreeItemIcon_Expanded);
 	SetItemImage(root, GetIconIndex(iconType::opened_dir), wxTreeItemIcon_SelectedExpanded);
 
+#if USE_MAC_SANDBOX
+	auto userDirs = OSXSandboxUserdirs::Get().GetDirs();
+	for (auto const& dir : userDirs) {
+		SetDir(dir);
+	}
+#endif
+
 	SetDir(_T("/"));
 #endif
 
@@ -288,26 +300,23 @@ CLocalTreeView::~CLocalTreeView()
 #endif
 }
 
-void CLocalTreeView::SetDir(wxString localDir)
+void CLocalTreeView::SetDir(wxString const& localDir)
 {
-	if (m_currentDir == localDir)
-	{
+	if (m_currentDir == localDir) {
 		RefreshListing();
 		return;
 	}
 
-	if (localDir.Left(2) == _T("\\\\"))
-	{
+	if (localDir.Left(2) == _T("\\\\")) {
 		// TODO: UNC path, don't display it yet
-		m_currentDir = _T("");
+		m_currentDir.clear();
 		SafeSelectItem(wxTreeItemId());
 		return;
 	}
 	m_currentDir = localDir;
 
 #ifdef __WXMSW__
-	if (localDir == _T("\\"))
-	{
+	if (localDir == _T("\\")) {
 		SafeSelectItem(m_drives);
 		return;
 	}
@@ -315,20 +324,19 @@ void CLocalTreeView::SetDir(wxString localDir)
 
 	wxString subDirs = localDir;
 	wxTreeItemId parent = GetNearestParent(subDirs);
-	if (!parent)
-	{
+	if (!parent) {
 		SafeSelectItem(wxTreeItemId());
 		return;
 	}
 
-	if (subDirs.empty())
-	{
+	if (subDirs.empty()) {
 		SafeSelectItem(parent);
 		return;
 	}
 	wxTreeItemId item = MakeSubdirs(parent, localDir.Left(localDir.Length() - subDirs.Length()), subDirs);
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	SafeSelectItem(item);
 }
@@ -443,8 +451,9 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, st
 		if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
 			if (!knownSubdir.empty()) {
 				wxTreeItemId item = GetSubdir(parent, knownSubdir);
-				if (item != wxTreeItemId())
+				if (item != wxTreeItemId()) {
 					return;
+				}
 
 				const wxString fullName = dirname + knownSubdir;
 				item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
@@ -456,20 +465,19 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, st
 					);
 				CheckSubdirStatus(item, fullName);
 			}
-			else
-			{
-				m_setSelection = true;
+			else {
+				++m_setSelection;
 				DeleteChildren(parent);
-				m_setSelection = false;
+				--m_setSelection;
 			}
 			return;
 		}
 	}
 
 	wxASSERT(parent);
-	m_setSelection = true;
+	++m_setSelection;
 	DeleteChildren(parent);
-	m_setSelection = false;
+	--m_setSelection;
 
 	CFilterManager filter;
 
@@ -483,11 +491,11 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, st
 	fz::datetime date;
 	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty()) {
+		std::wstring wfile = fz::to_wstring(file);
+		if (file.empty() || wfile.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
-		std::wstring wfile = fz::to_wstring(file);
 
 		std::wstring fullName = dirname.ToStdWstring() + wfile;
 #ifdef __WXMSW__
@@ -550,12 +558,12 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 	fz::datetime date;
 	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty()) {
+		std::wstring wfile = fz::to_wstring(file);
+		if (file.empty() || wfile.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
 
-		std::wstring wfile = fz::to_wstring(file);
 		if (filter.FilenameFiltered(wfile, dirname, true, size, true, attributes, date)) {
 			continue;
 		}
@@ -570,12 +578,16 @@ wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname,
 {
 	const wxString& separator = wxFileName::GetPathSeparator();
 
+	wxString segment;
 	while (!subDir.empty()) {
 		int pos = subDir.Find(separator);
-		wxString segment;
-		if (pos == -1) {
+		if (!pos) {
+			subDir = subDir.Mid(1);
+			continue;
+		}
+		else if (pos == -1) {
 			segment = subDir;
-			subDir = _T("");
+			subDir.clear();
 		}
 		else {
 			segment = subDir.Left(pos);
@@ -603,8 +615,10 @@ void CLocalTreeView::OnItemExpanding(wxTreeEvent& event)
 
 	wxTreeItemIdValue value;
 	wxTreeItemId child = GetFirstChild(item, value);
-	if (child && GetItemText(child).empty())
+	if (child && GetItemText(child).empty()) {
+		wxCHECK_RET(!m_setSelection, "OnItemExpanding called on an item with empty child during item selection of one of its children.");
 		DisplayDir(item, GetDirFromItem(item));
+	}
 }
 
 wxString CLocalTreeView::GetDirFromItem(wxTreeItemId item)
@@ -710,10 +724,10 @@ void CLocalTreeView::RefreshListing()
 		}
 	}
 #else
-	t_dir dir;
-	dir.dir = separator;
-	dir.item = GetRootItem();
-	dirsToCheck.push_back(dir);
+	t_dir root_dir;
+	root_dir.dir = separator;
+	root_dir.item = GetRootItem();
+	dirsToCheck.push_back(root_dir);
 #endif
 
 	CFilterManager filter;
@@ -747,12 +761,12 @@ void CLocalTreeView::RefreshListing()
 		int attributes;
 		fz::datetime date;
 		while (local_filesys.get_next_file(file, was_link, is_dir, 0, &date, &attributes)) {
-			if (file.empty()) {
+			std::wstring wfile = fz::to_wstring(file);
+			if (file.empty() || wfile.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			std::wstring wfile = fz::to_wstring(file);
 			if (filter.FilenameFiltered(wfile, dir.dir, true, size, true, attributes, date)) {
 				continue;
 			}
@@ -859,9 +873,9 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 		else {
 			wxBell();
 		}
-		m_setSelection = true;
+		++m_setSelection;
 		SelectItem(event.GetOldItem());
-		m_setSelection = false;
+		--m_setSelection;
 	}
 }
 
@@ -943,12 +957,16 @@ wxString CLocalTreeView::GetSpecialFolder(int folder, int &iconIndex, int &openI
 	if (SHGetSpecialFolderLocation((HWND)GetHandle(), folder, &list) == S_OK) {
 		SHFILEINFO shFinfo{};
 		if (SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON) != 0) {
-			DestroyIcon(shFinfo.hIcon);
+			if (shFinfo.hIcon) {
+				DestroyIcon(shFinfo.hIcon);
+			}
 			iconIndex = shFinfo.iIcon;
 		}
 
 		if (SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON | SHGFI_OPENICON | SHGFI_DISPLAYNAME) != 0) {
-			DestroyIcon(shFinfo.hIcon);
+			if (shFinfo.hIcon) {
+				DestroyIcon(shFinfo.hIcon);
+			}
 			openIconIndex = shFinfo.iIcon;
 			name = shFinfo.szDisplayName;
 		}
@@ -1042,10 +1060,24 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 		return;
 	}
 
-	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_LOCALTREE"));
-	if (!pMenu) {
-		return;
-	}
+	wxMenu menu;
+	auto item = new wxMenuItem(&menu, XRCID("ID_UPLOAD"), _("&Upload"), _("Upload selected directory"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOAD"), wxART_MENU));
+	menu.Append(item);
+	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
+	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU));
+	menu.Append(item);
+
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_OPEN"), _("&Open"), _("Open directory in system's file manager"));
+
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
+	menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
+
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_DELETE"), _("&Delete"), _("Delete selected directory"));
+	menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected directory"));
 
 	CLocalPath const path(GetDirFromItem(m_contextMenuItem).ToStdWstring());
 
@@ -1056,14 +1088,13 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 
 	bool const idle = m_state.GetLocalRecursiveOperation() && !m_state.GetLocalRecursiveOperation()->IsActive();
 
-	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected && idle);
-	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected && idle);
-	pMenu->Enable(XRCID("ID_MKDIR"), writeable);
-	pMenu->Enable(XRCID("ID_DELETE"), writeable && hasParent);
-	pMenu->Enable(XRCID("ID_RENAME"), writeable && hasParent);
+	menu.Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected && idle);
+	menu.Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected && idle);
+	menu.Enable(XRCID("ID_MKDIR"), writeable);
+	menu.Enable(XRCID("ID_DELETE"), writeable && hasParent);
+	menu.Enable(XRCID("ID_RENAME"), writeable && hasParent);
 
-	PopupMenu(pMenu);
-	delete pMenu;
+	PopupMenu(&menu);
 }
 
 void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
@@ -1087,7 +1118,11 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 		return;
 	}
 
-	const CServer server = *m_state.GetServer();
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
+		return;
+	}
+
 	CServerPath remotePath = m_state.GetRemotePath();
 	if (remotePath.empty()) {
 		return;
