@@ -19,10 +19,12 @@ CShellExtensionInterface::CShellExtensionInterface()
 		m_shellExtension = 0;
 	}
 
-	if (m_shellExtension)
+	if (m_shellExtension) {
 		m_hMutex = CreateMutex(0, false, _T("FileZilla3DragDropExtLogMutex"));
-	else
+	}
+	else {
 		m_hMutex = 0;
+	}
 
 	m_hMapping = 0;
 }
@@ -182,22 +184,18 @@ std::unique_ptr<CShellExtensionInterface> CShellExtensionInterface::CreateInitia
 	return ret;
 }
 
-//{7BB79969-2C7E-4107-996C-36DB90890AB2}
+#endif
 
-#endif //__WXMSW__
-
-CRemoteDataObject::CRemoteDataObject(const CServer& server, const CServerPath& path)
+CRemoteDataObject::CRemoteDataObject(ServerWithCredentials const& server, const CServerPath& path)
 	: wxDataObjectSimple(wxDataFormat(_T("FileZilla3RemoteDataObject")))
-	, m_server(server)
+	, server_(server)
 	, m_path(path)
-	, m_didSendData()
 	, m_processId(wxGetProcessId())
 {
 }
 
 CRemoteDataObject::CRemoteDataObject()
 	: wxDataObjectSimple(wxDataFormat(_T("FileZilla3RemoteDataObject")))
-	, m_didSendData()
 	, m_processId(wxGetProcessId())
 {
 }
@@ -236,65 +234,87 @@ void CRemoteDataObject::Finalize()
 
 	AddTextElement(element, "ProcessId", m_processId);
 
+	AddTextElement(element, "Count", m_fileList.size());
+
 	auto xServer = element.append_child("Server");
-	SetServer(xServer, m_server);
+	SetServer(xServer, server_);
 
 	AddTextElement(element, "Path", m_path.GetSafePath());
 
 	auto files = element.append_child("Files");
-	for (std::list<t_fileInfo>::const_iterator iter = m_fileList.begin(); iter != m_fileList.end(); ++iter) {
+	for (auto const& info : m_fileList) {
 		auto file = files.append_child("File");
-		AddTextElement(file, "Name", iter->name);
-		AddTextElement(file, "Dir", iter->dir ? 1 : 0);
-		AddTextElement(file, "Size", iter->size);
-		AddTextElement(file, "Link", iter->link ? 1 : 0);
+		AddTextElement(file, "Name", info.name);
+		AddTextElement(file, "Dir", info.dir ? 1 : 0);
+		AddTextElement(file, "Size", info.size);
+		AddTextElement(file, "Link", info.link ? 1 : 0);
 	}
 }
 
 bool CRemoteDataObject::SetData(size_t len, const void* buf)
 {
 	char* data = (char*)buf;
-	if (!len || data[len - 1] != 0)
+	if (!len || data[len - 1] != 0) {
 		return false;
+	}
 
-	if (!m_xmlFile.ParseData(data))
+	if (!m_xmlFile.ParseData(data)) {
 		return false;
+	}
 
 	auto element = m_xmlFile.GetElement();
-	if (!element || !(element = element.child("RemoteDataObject")))
+	if (!element || !(element = element.child("RemoteDataObject"))) {
 		return false;
+	}
 
 	m_processId = GetTextElementInt(element, "ProcessId", -1);
-	if (m_processId == -1)
+	if (m_processId == -1) {
 		return false;
+	}
 
-	auto server = element.child("Server");
-	if (!server || !::GetServer(server, m_server))
-		return false;
+	int64_t count = GetTextElementInt(element, "Count", -1);
+	if (count > 0) {
+		try {
+			m_fileList.reserve(static_cast<size_t>(count));
+		}
+		catch(std::exception const&) {
+			return false;
+		}
+	}
 
-	wxString path = GetTextElement(element, "Path");
-	if (path.empty() || !m_path.SetSafePath(path))
+	auto serverElement = element.child("Server");
+	if (!serverElement || !::GetServer(serverElement, server_)) {
 		return false;
+	}
+
+	std::wstring path = GetTextElement(element, "Path");
+	if (path.empty() || !m_path.SetSafePath(path)) {
+		return false;
+	}
 
 	m_fileList.clear();
 	auto files = element.child("Files");
-	if (!files)
+	if (!files) {
 		return false;
+	}
 
 	for (auto file = files.child("File"); file; file = file.next_sibling("File")) {
 		t_fileInfo info;
 		info.name = GetTextElement(file, "Name");
-		if (info.name.empty())
+		if (info.name.empty()) {
 			return false;
+		}
 
 		const int dir = GetTextElementInt(file, "Dir", -1);
-		if (dir == -1)
+		if (dir == -1) {
 			return false;
+		}
 		info.dir = dir == 1;
 
 		info.size = GetTextElementInt(file, "Size", -2);
-		if (info.size <= -2)
+		if (info.size <= -2) {
 			return false;
+		}
 
 		info.link = GetTextElementBool(file, "Link", false);
 
@@ -304,6 +324,11 @@ bool CRemoteDataObject::SetData(size_t len, const void* buf)
 	return true;
 }
 
+void CRemoteDataObject::Reserve(size_t count)
+{
+	m_fileList.reserve(count);
+}
+
 void CRemoteDataObject::AddFile(const wxString& name, bool dir, int64_t size, bool link)
 {
 	t_fileInfo info;
